@@ -66,14 +66,18 @@ _generator_opts = [
         'longer help text for Sphinx documents.'),
     cfg.StrOpt(
         'format',
-        help='Desired format for the output. "ini" is the only one which can '
-             'be used directly with oslo.config. "json" and "yaml" are '
-             'intended for third-party tools that want to write config files '
-             'based on the sample config data. "rst" can be used to dump '
-             'the text given to sphinx when building documentation using '
-             'the sphinx extension, for debugging.',
+        help='Desired format for the output.',
         default='ini',
-        choices=['ini', 'json', 'yaml', 'rst'],
+        choices=[
+            ('ini', 'The only format that can be used directly with '
+             'oslo.config.'),
+            ('json', 'Intended for third-party tools that want to write '
+             'config files based on the sample config data.'),
+            ('yaml', 'Same as json'),
+            ('rst', 'Can be used to dump the text given to Sphinx when '
+             'building documentation using the Sphinx extension. '
+             'Useful for debugging,')
+        ],
         dest='format_'),
 ]
 
@@ -161,14 +165,16 @@ class _OptFormatter(object):
 
     """Format configuration option descriptions to a file."""
 
-    def __init__(self, output_file=None, wrap_width=70):
+    def __init__(self, conf, output_file=None):
         """Construct an OptFormatter object.
 
+        :param conf: The config object from _generator_opts
         :param output_file: a writeable file object
-        :param wrap_width: The maximum length of help lines, 0 to not wrap
         """
         self.output_file = output_file or sys.stdout
-        self.wrap_width = wrap_width
+        self.wrap_width = conf.wrap_width
+        self.minimal = conf.minimal
+        self.summarize = conf.summarize
 
     def _format_help(self, help_text):
         """Format the help for a group or option to the output file.
@@ -213,13 +219,11 @@ class _OptFormatter(object):
             lines = ['[%s]\n' % groupname]
         self.writelines(lines)
 
-    def format(self, opt, group_name, minimal=False, summarize=False):
+    def format(self, opt, group_name):
         """Format a description of an option to the output file.
 
         :param opt: a cfg.Opt instance
         :param group_name: name of the group to which the opt is assigned
-        :param minimal: enable option by default, marking it as required
-        :param summarize: output a summarized description of the opt
         :returns: a formatted opt description string
         """
         if not opt.help:
@@ -234,7 +238,7 @@ class _OptFormatter(object):
         if opt.help:
             # an empty line signifies a new paragraph. We only want the
             # summary line
-            if summarize:
+            if self.summarize:
                 _split = opt.help.split('\n\n')
                 opt_help = _split[0].rstrip(':').rstrip('.')
                 if len(_split) > 1:
@@ -257,9 +261,12 @@ class _OptFormatter(object):
             lines.append('# Maximum value: %d\n' % opt.type.max)
 
         if getattr(opt.type, 'choices', None):
-            choices_text = ', '.join([self._get_choice_text(choice)
-                                      for choice in opt.type.choices])
-            lines.append('# Allowed values: %s\n' % choices_text)
+            lines.append('# Possible values:\n')
+            for choice in opt.type.choices:
+                help_text = '%s - %s' % (
+                    self._get_choice_text(choice),
+                    opt.type.choices[choice] or '<No description provided>')
+                lines.extend(self._format_help(help_text))
 
         try:
             if opt.mutable:
@@ -314,6 +321,14 @@ class _OptFormatter(object):
                 '# effect on stability and/or performance.\n'
             )
 
+        if opt.sample_default:
+            lines.append(
+                '#\n'
+                '# This option has a sample default set, which means that\n'
+                '# its actual default value may vary from the one documented\n'
+                '# below.\n'
+            )
+
         if hasattr(opt.type, 'format_defaults'):
             defaults = opt.type.format_defaults(opt.default,
                                                 opt.sample_default)
@@ -328,7 +343,7 @@ class _OptFormatter(object):
         for default_str in defaults:
             if default_str:
                 default_str = ' ' + default_str.replace('\n', '\n#    ')
-            if minimal:
+            if self.minimal:
                 lines.append('%s =%s\n' % (opt.dest, default_str))
             else:
                 lines.append('#%s =%s\n' % (opt.dest, default_str))
@@ -513,18 +528,18 @@ def on_load_failure_callback(*args, **kwargs):
     raise
 
 
-def _output_opts(f, group, group_data, minimal=False, summarize=False):
+def _output_opts(f, group, group_data):
     f.format_group(group_data['object'] or group)
     for (namespace, opts) in sorted(group_data['namespaces'],
                                     key=operator.itemgetter(0)):
         f.write('\n#\n# From %s\n#\n' % namespace)
         for opt in sorted(opts, key=operator.attrgetter('advanced')):
             try:
-                if minimal and not opt.required:
+                if f.minimal and not opt.required:
                     pass
                 else:
                     f.write('\n')
-                    f.format(opt, group, minimal, summarize)
+                    f.format(opt, group)
             except Exception as err:
                 f.write('# Warning: Failed to format sample for %s\n' %
                         (opt.dest,))
@@ -581,9 +596,14 @@ def _build_entry(opt, group, namespace, conf):
     entry = {key: value for key, value in opt.__dict__.items()
              if not key.startswith('_')}
     entry['namespace'] = namespace
-    # In some types, choices is explicitly set to None.  Force it to [] so it
-    # is always an iterable type.
-    entry['choices'] = getattr(entry['type'], 'choices', []) or []
+    # Where present, we store choices as an OrderedDict. The default repr for
+    # this is not very machine readable, thus, it is switched to a list of
+    # tuples here. In addition, in some types, choices is explicitly set to
+    # None. Force these cases to [] so it is always an iterable type.
+    if getattr(entry['type'], 'choices', None):
+        entry['choices'] = list(entry['type'].choices.items())
+    else:
+        entry['choices'] = []
     entry['min'] = getattr(entry['type'], 'min', None)
     entry['max'] = getattr(entry['type'], 'max', None)
     entry['type'] = _format_type_name(entry['type'])
@@ -712,25 +732,27 @@ def generate(conf, output_file=None):
     """
     conf.register_opts(_generator_opts)
 
+    own_file = False
+
     if output_file is None:
-        output_file = (open(conf.output_file, 'w')
-                       if conf.output_file else sys.stdout)
+        if conf.output_file:
+            output_file = open(conf.output_file, 'w')
+            own_file = True
+        else:
+            output_file = sys.stdout
 
     groups = _get_groups(_list_opts(conf.namespace))
 
     if conf.format_ == 'ini':
-        formatter = _OptFormatter(output_file=output_file,
-                                  wrap_width=conf.wrap_width)
+        formatter = _OptFormatter(conf, output_file=output_file)
 
         # Output the "DEFAULT" section as the very first section
-        _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'), conf.minimal,
-                     conf.summarize)
+        _output_opts(formatter, 'DEFAULT', groups.pop('DEFAULT'))
 
         # output all other config sections with groups in alphabetical order
         for group, group_data in sorted(groups.items()):
             formatter.write('\n\n')
-            _output_opts(formatter, group, group_data, conf.minimal,
-                         conf.summarize)
+            _output_opts(formatter, group, group_data)
     elif conf.format_ == 'rst':
         _output_human_readable(
             conf.namespace,
@@ -741,6 +763,9 @@ def generate(conf, output_file=None):
                                  output_file=output_file,
                                  conf=conf)
 
+    if own_file:
+        output_file.close()
+
 
 def main(args=None):
     """The main function of oslo-config-generator."""
