@@ -44,7 +44,10 @@ class ConfigType(object):
 
         """
         if sample_default is not None:
-            default_str = sample_default
+            if isinstance(sample_default, six.string_types):
+                default_str = sample_default
+            else:
+                default_str = self._formatter(sample_default)
         elif default is None:
             default_str = self.NONE_DEFAULT
         else:
@@ -348,7 +351,7 @@ class Number(ConfigType):
         )
 
     def _formatter(self, value):
-        return str(value)
+        return six.text_type(value)
 
 
 class Integer(Number):
@@ -458,7 +461,8 @@ class List(ConfigType):
     comma and next item until validation succeeds or there is no parts left.
     In the later case it will signal validation error.
 
-    :param item_type: type of list items
+    :param item_type: Type of list items. Should be an instance of
+                      ``ConfigType``.
     :param bounds: if True, value should be inside "[" and "]" pair
     :param type_name: Type name to be used in the sample config file.
 
@@ -531,10 +535,11 @@ class List(ConfigType):
             return fmtstr.format(value)
         if isinstance(value, list):
             value = [
-                six.text_type(v)
+                self.item_type._formatter(v)
                 for v in value
             ]
-        return fmtstr.format(','.join(value))
+            return fmtstr.format(','.join(value))
+        return fmtstr.format(self.item_type._formatter(value))
 
 
 class Range(ConfigType):
@@ -879,20 +884,23 @@ class URI(ConfigType):
         self.schemes = schemes
 
     def __call__(self, value):
-        if not rfc3986.is_valid_uri(value, require_scheme=True,
-                                    require_authority=True):
-            raise ValueError('invalid URI: %r' % value)
+        uri = rfc3986.uri_reference(value)
+        validator = rfc3986.validators.Validator().require_presence_of(
+            'scheme', 'host',
+        ).check_validity_of(
+            'scheme', 'host', 'path',
+        )
+        if self.schemes:
+            validator = validator.allow_schemes(*self.schemes)
+        try:
+            validator.validate(uri)
+        except rfc3986.exceptions.RFC3986Exception as exc:
+            raise ValueError(exc)
 
         if self.max_length is not None and len(value) > self.max_length:
             raise ValueError("Value '%s' exceeds maximum length %d" %
                              (value, self.max_length))
 
-        if self.schemes:
-            scheme = rfc3986.uri_reference(value).scheme
-            if scheme not in self.schemes:
-                raise ValueError("URI scheme '%s' not in %s" %
-                                 (scheme, self.schemes))
-
         # NOTE(dhellmann): self.value is deprecated, and we don't want
         # to trigger a deprecation warning ourselves so we modify
         # self._value directly.
